@@ -1,6 +1,6 @@
 package Mojo::Webqq;
 use strict;
-$Mojo::Webqq::VERSION = "1.4.4";
+$Mojo::Webqq::VERSION = "1.4.5";
 use base qw(Mojo::Base);
 use Mojo::Webqq::Log;
 use Mojo::Webqq::Cache;
@@ -119,4 +119,21 @@ sub on {
     return $self;
 }
 
+sub friends{
+    my $self = shift;
+    return @{$self->friend};
+}
+sub groups{
+    my $self = shift;
+    return @{$self->group};
+}
+sub discusss{
+    my $self = shift;
+    return @{$self->discuss};
+}
+sub recents{
+    my $self = shift;
+    return @{$self->recent};
+}
+
 1;
